/*
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.

 * Contributor(s) Name, Email, Jabber ID: 
 *
 *  Eric Murphy, ericmurphy@jabber.org, ericmurphy@jabber.org

 * Summary:
 *
 *  This handles all the message handling of Jabberzilla for JabberXM.

 * Change Log: 
 *
 *  2002.09.22:
 *
 *    1. Created file.

 *  2002.11.24:
 *
 *    1. Reimplemented chat support.
 */

function Message()
{
  this.jabberXMConstructor = 
         new Components.Constructor("@jabberxm.mozdev.org;1","JabberXM");
  this.jabberXM = new this.jabberXMConstructor();

  this.windowMediatorService =
         Components.classes["@mozilla.org/appshell/window-mediator;1"].
           getService(Components.interfaces.nsIWindowMediator);

  this.wrappedJSObject = this;
  this.postHandler = null;
  // Array of namespaces each containing an array of handlers for that namespace.
  this.nsHandler = new Array(); 
}

Message.prototype.setChatHandler = function(chatHandler)
{
  this.chathandler = chatHandlerchatHandler;
}

Message.prototype.setMessageHandler = function(messageHandler)
{
  this.messagehandler = messageHandler.messageHandler;
}

Message.prototype.handleChat = function(jid,subject,body,html)
{
  var xulWindows = this.windowMediatorService.getZOrderDOMWindowEnumerator("jabberzilla:chat",true);
  var hasMoreElements = true;
  var foundWindow;

  while(hasMoreElements)
  {
    if(xulWindows.hasMoreElements())
    {
      var xulWindow = xulWindows.getNext();
      xulWindow.QueryInterface(Components.interfaces.nsIDOMWindow);
      xulWindow = xulWindow;

      if(xulWindow.name==jid)
      {
        foundWindow = xulWindow;
        break;
      }
    }
    else
    { 
      foundWindow = window.openDialog('chrome://jabberzilla/content/chat.xul',jid,'chrome,all,dialog=no');
      foundWindow.firstMessage = { jid:jid,subject:subject,body:body,html:html };

      foundWindow.name = jid;
      hasMoreElements = false;
    }
  }

  foundWindow.getAttention();
  foundWindow.incomingMessage(jid,subject,body,html);
}

Message.prototype.openChat = function(jid)
{
  var xulWindows = this.windowMediatorService.getXULWindowEnumerator("navigator:browser");
  var hasMoreElements = true;

  while(hasMoreElements)
  {
    if(xulWindows.hasMoreElements())
    {
      var xulWindow = xulWindows.getNext();
      xulWindow.QueryInterface(Components.interfaces.nsIXULWindow);

      if(xulWindow.name==jid)
        break;
    }
    else
    {
      window.openDialog('chrome://jabberzilla/content/chat.xul',jid,'chrome,all,dialog=no');
      hasMoreElements = false;
    }
  }
}

Message.prototype.closeChat = function(jid)
{
  JabberXMModule.chatWindows[jid] = null;
}

Message.prototype.openMessage = function(jid)
{
  if(JabberXMModule.messageWindows[jid] == null)
  {
    JabberXMModule.messageWindows[jid] = 
      this.messageWindow.open(this.messageWindowUri,jid,this.messageAttributes);

    JabberXMModule.messageWindows[jid].messageHandler = this.messageHandler;
  }
}

Message.prototype.messageChat = function(jid)
{
  JabberXMModule.chatWindows[jid] = null;
}

Message.prototype.packetMessage = function(packet)
{
  var messageType = '';

  switch( packet.attribs["type"] )
  {
    case 'chat':
      messageType = 'chat';
      break;
    case 'groupchat':
      messageType = 'groupchat';
      break;
    case 'error':
      messageType = 'error';
      break;
    default:
      messageType = 'normal';
  }

  var subject=''; var body=''; var thread=''; var html=''; var w='';

  var calledHandler = false;

  for(var i = 0; i < packet.children.length; i++)
  {
    var child = packet.children[i];
    switch( child.name )
    {
      case "subject":
        subject = child.cdata;
        break;

      case "body":
        body = child.cdata;
        break;

      case "thread":
        thread = child.cdata;
        break;

      case "html":
        var currentData = JabberXMModule.socket.getData();
        var beginning = currentData.search("<html");
        html = currentData.substring(beginning);
        var end = html.search("</html>");
        html = html.substring(0,end+7);
        break;

      case "w":
        var currentData = JabberXMModule.socket.getData();
        var beginning = currentData.search("<w");
        w = currentData.substring(beginning);
        var end = w.search("</w>");
        w = w.substring(0,end+4);
        break;

      case "x":
        var xmlns = child.attribs["xmlns"];
        if( this.nsHandler[ xmlns ] )
        {
          var handler = this.nsHandler[ xmlns ];
          for( var i = 0; i < handler.length; i++ )
          {
            handler[i].handleMessage( packet );
            calledHandler = true;
          }
        }
        else
        {
          dump( "No handler registered for namespace " + xmlns + "." );
        }
        break;

      default:
        dump( "unknown message element: " + child.name );
    }
  }

  if(calledHandler)
    return;

  var fulljid =packet.attribs["from"];
  var jid = packet.attribs["from"] + "/";
  jid = jid.substr( 0, jid.indexOf( "/" ) );

  if(messageType=='normal' || messageType=='chat')
  {
    //this.handleMessage(jid,subject,body);
    this.handleChat(jid,subject,body,html);
  }
  else if(messageType=='groupchat')
  {
    if(w != '')
    {
      name = JabberXMModule.conferenceUserNameList[fulljid];
      this.whiteboardHandler.handleWhiteboard(name,body,w);
    }
    else if(JabberXMModule.conferenceChatHandlers[jid] != null)
    {
      name = JabberXMModule.conferenceUserNameList[fulljid];
      if(name==null)
       name='';

      JabberXMModule.conferenceChatHandlers[jid](name,subject,body,html);
    }
    else if(this.postHandler != null)
    {
      this.postHandler.handlePost(jid,subject,body,thread);
    }
  }
  else
  {

  }
}

Message.prototype.registerNSHandler = function(namespace,handler)
{
  var newHandler = handler.wrappedJSObject;
  if( this.nsHandler[ namespace ] )
  {
    if( this.nsHandler[ namespace ][ newHandler ] )
    {
      dump( "Handler already registered." );
      return false;
    }
    else
      this.nsHandler[ namespace ].push( newHandler );
  }
  else
  {
    this.nsHandler[ namespace ] = new Array( newHandler );
  }
  return true;
}

Message.prototype.unregisterNSHandler = function(namespace,handler)
{
  var oldHandler = handler.wrappedJSObject;
  if( this.nsHandler[ namespace ] )
  {
    for( var i = 0; i < this.nsHandler[ namespace ].length; i++ )
    {
      if( this.nsHandler[ namespace ][ i ] == oldHandler )
      {
        this.nsHandler[ namespace ].splice( i, 1 );
        if( this.nsHandler[ namespace ].length == 0 )
        {
         //TODO: delete the namespace entry - something like this:
         //this.nsHandler[ namespace ] = undefined;
        }
        return true;
      }
    }
  }
  return false;
}