/*
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.

 * Contributor(s) Name, Email, Jabber ID: 
 *
 *  Eric Murphy, ericmurphy@jabber.org, ericmurphy@jabber.org

 * Summary:
 *
 *  This handles all the presence handling of Jabberzilla for JabberXM.

 * Change Log: 
 *
 *  2002.09.22:
 *
 *    1. Created file.
 */

function Presence()
{
  this.wrappedJSObject = this;
  this.presence = new this.PresenceData('http://jabberxm.mozdev.org/rdf#groups');
  this.showResource  = 'http://jabberxm.mozdev.org/rdf#show';
  this.statusResource = 'http://jabberxm.mozdev.org/rdf#status';
  this.presenceLoaded = false;
}

Presence.prototype.sendPresence = function(show, status)
{
  if (show == 'available')
    JabberXMModule.socket.write('<presence/>');

  else
  {
    JabberXMModule.socket.write
    (
      '<presence><show>' + 
       show  + '</show><status>' + 
       status + '</status></presence>'
    );
  }
}

Presence.prototype.changeContactPresence = function(jid,res,show,status)
{
  this.presenceLoaded = true;

  if(res!='')
    var user = this.presence.rdf.GetResource(jid+'/'+res);
  else
    var user = this.presence.rdf.GetResource(jid);

  target = 
    this.presence.datasource.GetTarget
    (
      user,
      this.presence.rdf.GetResource(this.showResource), 
      true
    );

  if(target==null)
  {
    this.presence.datasource.Assert
    (
      user,
      this.presence.rdf.GetResource(this.showResource),
      this.presence.rdf.GetLiteral(show),
      true
    );
  }
  else if ( JabberXMModule.hasReceivedRoster )
  {
    this.presence.datasource.Change
    (
      user,
      this.presence.rdf.GetResource(this.showResource),
      target,
      this.presence.rdf.GetLiteral(show)
    );
  }

  target = 
    this.presence.datasource.GetTarget
    (
      user,
      this.presence.rdf.GetResource(this.statusResource),
      true
    );

  if ( target == null )
  {
    this.presence.datasource.Assert
    (
      user,
      this.presence.rdf.GetResource(this.statusResource),
      this.presence.rdf.GetLiteral(status),
      true
    );
  }
  else if ( JabberXMModule.hasReceivedRoster )
  {
    this.presence.datasource.Change
    (
      user,
      this.presence.rdf.GetResource(this.statusResource),
      target,
      this.presence.rdf.GetLiteral(status)
    );
  }

  this.presence.sequence.AppendElement(user);

  if(res!='') {
    res='';
    this.changeContactPresence(jid,res,show,status);
  }
}

/*
  this.presence.datasource.Unassert
  (
    user,
    this.presence.rdf.GetResource(this.showResource),
    this.presence.rdf.GetLiteral(show)
  );
*/

Presence.prototype.packetPresence = function(packet)
{
  // Get the JID and Resource (hack)
  p = '';  q = 0;

  if(packet.attribs["type"]=='error')
  {}
  else if(packet.attribs["type"]=='subscribe')
  {
    jid = packet.attribs["from"]; status = '';

    for(var i = 0; i < packet.children.length; i++){
      if(packet.children[i].name == "status")
        status = packet.children[i].cdata;
    }
    this.presenceRequestWindow.open
    (this.presenceRequestWindowUri,jid,this.presenceRequestAttributes);
  }

  else if (packet.attribs["from"]!=null)
  {
    from = packet.attribs["from"];

    // Hack off resource, except for Agents

    if (from.search('icq')!=-1 && from.search('@icq')==-1)
    {
      jid = from; res = '';
    }
    else if (from.search('aim')!=-1 && from.search('@aim')==-1)
    {
      jid = from; res = '';
    }
    else if (from.search('msn')!=-1 && from.search('@msn')==-1)
    {
      jid = from; res = '';
    }
    else if (from.search('yahoo')!=-1 && from.search('@yahoo')==-1)
    {
      jid = from; res = '';
    }
    else
    {
      from=packet.attribs["from"] + '/';

      while(p != '/')
      {
        p=from[q++];
      }

      jid=(from.slice(0,q-1));

      res=(from.slice(q,from.length-1));
    }
  }

  // Get the Priority
  for(var i = 0; i < packet.children.length; i++)
  {
    if(packet.children[i].name == "priority")
    {
      priority = packet.children[i].cdata;
    }
  }

  // Default
  show = 'available';

  if(packet.attribs["type"]=='error')
  {
    show = 'unknown';
  }

  if (packet.attribs["type"]=='unavailable')
  {
    show = 'unavailable';
  }

  for(i=0;i<packet.children.length;i++)
  {
    if(packet.children[i].name=="show")
    {
      if (packet.children[i].cdata != '')
        show = packet.children[i].cdata;
    }
  }

  if(show == 'available')
    status = 'Online';
  else
    status = '';

  for(var i = 0; i < packet.children.length; i++)
  {
    if(packet.children[i].name == "status")
    {
      status = packet.children[i].cdata;
    }
  }

  this.changeContactPresence(jid,res,show,status);
}

Presence.prototype.PresenceData = function(uri)
{
  this.rdf = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService(Components.interfaces.nsIRDFService);

  this.datasource = Components.classes['@mozilla.org/rdf/datasource;1?name=in-memory-datasource'].createInstance(Components.interfaces.nsIRDFDataSource);

  this.root = this.rdf.GetResource('urn:root');
  this.anon = this.rdf.GetAnonymousResource();

  this.datasource.Assert(this.root,this.rdf.GetResource(uri),this.anon,true);

  this.rdfc = Components.classes['@mozilla.org/rdf/container-utils;1'].getService(Components.interfaces.nsIRDFContainerUtils);

  this.sequence = this.rdfc.MakeSeq(this.datasource,this.anon);

  this.rdfs = Components.classes['@mozilla.org/rdf/container;1'].getService(Components.interfaces.nsIRDFContainer);
}