/*
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.

 * Contributor(s) Name, Email, Jabber ID: 
 *
 *   Eric Murphy, ericmurphy@jabber.org, ericmurphy@jabber.org

 * Summary:
 *
 *   This handles all the session management of Jabberzilla for JabberXM.

 * Change Log: 
 *
 *   2002.09.22:
 *
 *     1. Created file.
 */

function Session()
{
  this.jabberXMConstructor = 
         new Components.Constructor("@jabberxm.mozdev.org;1","JabberXM");
  this.jabberXM = new this.jabberXMConstructor();

  this.wrappedJSObject = this;
  this.id = ''; // session id for SHA-1 digest
  this.hex_chr = "0123456789abcdef";
}

Session.prototype.packetSession = function(packet)
{
  this.id = packet.attribs["id"];

  if(!this.jabberXM.isAuth()==true)
  {
    this.authenticate();
    this.jabberXM.requestRoster();
  }
  else
    this.jabberXM.register();
}

Session.prototype.authenticate = function()
{
  var digest = this.id + this.jabberXM.getPassword();
  digest = this.calcSHA1(digest);

  var type = "digest";
  this.jabberXM.authenticate(type,digest);
}

/**********************************************************************************/
// This created the hash used for the password on the login of the client.

Session.prototype.hex = function(num)
{
   var str = "";
   for(var j = 7; j >= 0; j--)
      str += this.hex_chr.charAt((num >> (j * 4)) & 0x0F);
   return str;
}

Session.prototype.str2blks_SHA1 = function(str) 
{
  var nblk = ((str.length + 8) >> 6) + 1;
  var blks = new Array(nblk * 16);
  for(var i = 0; i < nblk * 16; i++) blks[i] = 0;
  for(i = 0; i < str.length; i++)
    blks[i >> 2] |= str.charCodeAt(i) << (24 - (i % 4) * 8);
  blks[i >> 2] |= 0x80 << (24 - (i % 4) * 8);
  blks[nblk * 16 - 1] = str.length * 8;
  return blks;
}

Session.prototype.add = function(x,y)
{
  return ((x&0x7FFFFFFF) + (y&0x7FFFFFFF)) ^ (x&0x80000000) ^ (y&0x80000000);
}

Session.prototype.rol = function(num,cnt)
{
  return (num << cnt) | (num >>> (32 - cnt));
}

Session.prototype.ft = function(t,b,c,d)
{
  if(t < 20) return (b & c) | ((~b) & d);
  if(t < 40) return b ^ c ^ d;
  if(t < 60) return (b & c) | (b & d) | (c & d);
  return b ^ c ^ d;
}

Session.prototype.kt = function(t)
{
  if(t < 20) return 0x5A827999;
  if(t < 40) return 0x6ED9EBA1;
  if(t < 60) return 0x8F1BBCDC;
  return 0xCA62C1D6;
}

Session.prototype.calcSHA1 = function(str)
{
  var x = this.str2blks_SHA1(str);
  var w = new Array(80);

  var a = 0x67452301;
  var b = 0xEFCDAB89;
  var c = 0x98BADCFE;
  var d = 0x10325476;
  var e = 0xC3D2E1F0;

  for(var i = 0; i < x.length; i += 16)
  {
    var olda = a;
    var oldb = b;
    var oldc = c;
    var oldd = d;
    var olde = e;

    for(var j = 0; j < 80; j++)
    {
      if(j < 16) w[j] = x[i + j];
      else w[j] = this.rol(w[j-3] ^ w[j-8] ^ w[j-14] ^ w[j-16], 1);
      t = this.add(this.add(this.rol(a, 5), this.ft(j, b, c, d)), this.add(this.add(e, w[j]), this.kt(j)));
      e = d;
      d = c;
      c = this.rol(b, 30);
      b = a;
      a = t;
    }
    a = this.add(a, olda);
    b = this.add(b, oldb);
    c = this.add(c, oldc);
    d = this.add(d, oldd);
    e = this.add(e, olde);
  }
  return this.hex(a) + this.hex(b) + this.hex(c) + this.hex(d) + this.hex(e);
}